# Generated by Django 3.0.8 on 2020-09-01 14:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_opstasks', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='taskresult',
            name='date_start',
            field=models.DateTimeField(auto_now_add=True, null=True, verbose_name='Started DateTime'),
        ),
        migrations.AddField(
            model_name='taskresult',
            name='queue',
            field=models.CharField(blank=True, max_length=155, null=True, verbose_name='Queue'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='content_encoding',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='Result Encoding'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='content_type',
            field=models.CharField(blank=True, max_length=128, null=True, verbose_name='Result Content Type'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Created DateTime'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='date_done',
            field=models.DateTimeField(auto_now=True, db_index=True, verbose_name='Completed DateTime'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='meta',
            field=models.TextField(default=None, editable=False, null=True, verbose_name='Task Meta Information'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='result',
            field=models.TextField(blank=True, default=None, editable=False, null=True, verbose_name='Result Data'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='status',
            field=models.CharField(choices=[('FAILURE', 'FAILURE'), ('PENDING', 'PENDING'), ('RECEIVED', 'RECEIVED'), ('RETRY', 'RETRY'), ('REVOKED', 'REVOKED'), ('STARTED', 'STARTED'), ('SUCCESS', 'SUCCESS')], db_index=True, default='PENDING', max_length=50, verbose_name='Task State'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='task_args',
            field=models.TextField(null=True, verbose_name='Task Positional Arguments'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='task_id',
            field=models.CharField(db_index=True, max_length=255, unique=True, verbose_name='Task ID'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='task_kwargs',
            field=models.TextField(null=True, verbose_name='Task Named Arguments'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='task_name',
            field=models.CharField(db_index=True, max_length=255, null=True, verbose_name='Task Name'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='traceback',
            field=models.TextField(blank=True, null=True, verbose_name='Traceback'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='worker',
            field=models.CharField(db_index=True, default=None, max_length=100, null=True, verbose_name='Worker'),
        ),
    ]
