# Generated by Django 3.0.8 on 2020-09-01 14:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_opstasks', '0002_auto_20200901_2215'),
    ]

    operations = [
        migrations.AlterField(
            model_name='taskresult',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, db_index=True, null=True, verbose_name='Created DateTime'),
        ),
        migrations.AlterField(
            model_name='taskresult',
            name='date_done',
            field=models.DateTimeField(auto_now=True, db_index=True, null=True, verbose_name='Completed DateTime'),
        ),
    ]
