# Generated by Django 3.0.8 on 2020-09-10 07:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_opstasks', '0003_auto_20200901_2231'),
    ]

    operations = [
        migrations.CreateModel(
            name='TaskStartDatetime',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('task_id', models.CharField(db_index=True, max_length=255, unique=True, verbose_name='Task ID')),
                ('date_start', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Started DateTime')),
            ],
            options={
                'db_table': 'tasks_start_datetime',
            },
        ),
    ]
